@@ -54,4 +54,10 @@
 #define LTTNG_MAX_PID				INT32_MAX
 #define LTTNG_MAX_PID_STR			"2147483647"
 
+#define LTTNG_NAME_MAX				255
+
+#define LTTNG_HOST_NAME_MAX			64
+
+#define LTTNG_PATH_MAX				4096
+
 #endif /* LTTNG_CONSTANT_H */
