@@ -24,6 +24,7 @@
 
 #include <lttng/constant.h>
 #include <common/macros.h>
+#include <common/config/session-config.h>
 
 /*
  * Object used by the load_session API. This is opaque to the public library.
@@ -35,6 +36,16 @@ struct lttng_load_session_attr {
 	char input_url[PATH_MAX];
 	/* Overwrite the session if it exists. */
 	uint32_t overwrite;
+	/* The raw override url for getter */
+	char *raw_override_url;
+	/* The raw override path url for getter */
+	char *raw_override_path_url;
+	/* The raw override ctrl url for getter */
+	char *raw_override_ctrl_url;
+	/* The raw override data url for getter */
+	char *raw_override_data_url;
+	/* Override struct */
+	struct config_load_session_override_attr *override_attr;
 } LTTNG_PACKED;
 
 #endif /* LTTNG_LOAD_INTERNAL_ABI_H */
