@@ -748,7 +748,6 @@ int consumer_send_fds(struct consumer_socket *sock, int *fds, size_t nb_fd)
 	}
 
 	ret = consumer_recv_status_reply(sock);
-
 error:
 	return ret;
 }
@@ -806,6 +805,7 @@ void consumer_init_ask_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 		unsigned int switch_timer_interval,
 		unsigned int read_timer_interval,
 		unsigned int live_timer_interval,
+		unsigned int monitor_timer_interval,
 		int output,
 		int type,
 		uint64_t session_id,
@@ -822,6 +822,7 @@ void consumer_init_ask_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 		uint64_t session_id_per_pid,
 		unsigned int monitor,
 		uint32_t ust_app_uid,
+		int64_t blocking_timeout,
 		const char *root_shm_path,
 		const char *shm_path)
 {
@@ -837,6 +838,7 @@ void consumer_init_ask_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 	msg->u.ask_channel.switch_timer_interval = switch_timer_interval;
 	msg->u.ask_channel.read_timer_interval = read_timer_interval;
 	msg->u.ask_channel.live_timer_interval = live_timer_interval;
+	msg->u.ask_channel.monitor_timer_interval = monitor_timer_interval;
 	msg->u.ask_channel.output = output;
 	msg->u.ask_channel.type = type;
 	msg->u.ask_channel.session_id = session_id;
@@ -850,6 +852,7 @@ void consumer_init_ask_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 	msg->u.ask_channel.tracefile_count = tracefile_count;
 	msg->u.ask_channel.monitor = monitor;
 	msg->u.ask_channel.ust_app_uid = ust_app_uid;
+	msg->u.ask_channel.blocking_timeout = blocking_timeout;
 
 	memcpy(msg->u.ask_channel.uuid, uuid, sizeof(msg->u.ask_channel.uuid));
 
@@ -892,7 +895,8 @@ void consumer_init_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 		uint64_t tracefile_size,
 		uint64_t tracefile_count,
 		unsigned int monitor,
-		unsigned int live_timer_interval)
+		unsigned int live_timer_interval,
+		unsigned int monitor_timer_interval)
 {
 	assert(msg);
 
@@ -913,6 +917,7 @@ void consumer_init_channel_comm_msg(struct lttcomm_consumer_msg *msg,
 	msg->u.channel.tracefile_count = tracefile_count;
 	msg->u.channel.monitor = monitor;
 	msg->u.channel.live_timer_interval = live_timer_interval;
+	msg->u.channel.monitor_timer_interval = monitor_timer_interval;
 
 	strncpy(msg->u.channel.pathname, pathname,
 			sizeof(msg->u.channel.pathname));
@@ -1048,6 +1053,35 @@ error:
 	return ret;
 }
 
+int consumer_send_channel_monitor_pipe(struct consumer_socket *consumer_sock,
+		int pipe)
+{
+	int ret;
+	struct lttcomm_consumer_msg msg;
+
+	/* Code flow error. Safety net. */
+
+	memset(&msg, 0, sizeof(msg));
+	msg.cmd_type = LTTNG_CONSUMER_SET_CHANNEL_MONITOR_PIPE;
+
+	DBG3("Sending set_channel_monitor_pipe command to consumer");
+	ret = consumer_send_msg(consumer_sock, &msg);
+	if (ret < 0) {
+		goto error;
+	}
+
+	DBG3("Sending channel monitoring pipe %d to consumer on socket %d",
+			pipe, *consumer_sock->fd_ptr);
+	ret = consumer_send_fds(consumer_sock, &pipe, 1);
+	if (ret < 0) {
+		goto error;
+	}
+
+	DBG2("Channel monitoring pipe successfully sent");
+error:
+	return ret;
+}
+
 /*
  * Set consumer subdirectory using the session name and a generated datetime if
  * needed. This is appended to the current subdirectory.
