@@ -36,6 +36,8 @@
 
 /*
  * Add context on a kernel channel.
+ *
+ * Assumes the ownership of ctx.
  */
 int kernel_add_channel_context(struct ltt_kernel_channel *chan,
 		struct ltt_kernel_context *ctx)
@@ -63,12 +65,16 @@ int kernel_add_channel_context(struct ltt_kernel_channel *chan,
 			goto error;
 		}
 	}
+	ret = 0;
 
 end:
 	cds_list_add_tail(&ctx->list, &chan->ctx_list);
-	return 0;
-
+	ctx->in_list = true;
+	ctx = NULL;
 error:
+	if (ctx) {
+		trace_kernel_destroy_context(ctx);
+	}
 	return ret;
 }
 
@@ -644,17 +650,22 @@ error:
  * Open stream of channel, register it to the kernel tracer and add it
  * to the stream list of the channel.
  *
+ * Note: given that the streams may appear in random order wrt CPU
+ * number (e.g. cpu hotplug), the index value of the stream number in
+ * the stream name is not necessarily linked to the CPU number.
+ *
  * Return the number of created stream. Else, a negative value.
  */
 int kernel_open_channel_stream(struct ltt_kernel_channel *channel)
 {
-	int ret, count = 0;
+	int ret;
 	struct ltt_kernel_stream *lks;
 
 	assert(channel);
 
 	while ((ret = kernctl_create_stream(channel->fd)) >= 0) {
-		lks = trace_kernel_create_stream(channel->channel->name, count);
+		lks = trace_kernel_create_stream(channel->channel->name,
+				channel->stream_count);
 		if (lks == NULL) {
 			ret = close(ret);
 			if (ret) {
@@ -673,13 +684,10 @@ int kernel_open_channel_stream(struct ltt_kernel_channel *channel)
 		lks->tracefile_size = channel->channel->attr.tracefile_size;
 		lks->tracefile_count = channel->channel->attr.tracefile_count;
 
-		/* Add stream to channe stream list */
+		/* Add stream to channel stream list */
 		cds_list_add(&lks->list, &channel->stream_list.head);
 		channel->stream_count++;
 
-		/* Increment counter which represent CPU number. */
-		count++;
-
 		DBG("Kernel stream %s created (fd: %d, state: %d)", lks->name, lks->fd,
 				lks->state);
 	}
@@ -1086,3 +1094,35 @@ int kernel_syscall_mask(int chan_fd, char **syscall_mask, uint32_t *nr_bits)
 
 	return kernctl_syscall_mask(chan_fd, syscall_mask, nr_bits);
 }
+
+/*
+ * Check for the support of the RING_BUFFER_SNAPSHOT_SAMPLE_POSITIONS via abi
+ * version number.
+ *
+ * Return 1 on success, 0 when feature is not supported, negative value in case
+ * of errors.
+ */
+int kernel_supports_ring_buffer_snapshot_sample_positions(int tracer_fd)
+{
+	int ret = 0; // Not supported by default
+	struct lttng_kernel_tracer_abi_version abi;
+
+	ret = kernctl_tracer_abi_version(tracer_fd, &abi);
+	if (ret < 0) {
+		ERR("Failed to retrieve lttng-modules ABI version");
+		goto error;
+	}
+
+	/*
+	 * RING_BUFFER_SNAPSHOT_SAMPLE_POSITIONS was introduced in 2.3
+	 */
+	if (abi.major >= 2 && abi.minor >= 3) {
+		/* Supported */
+		ret = 1;
+	} else {
+		/* Not supported */
+		ret = 0;
+	}
+error:
+	return ret;
+}
