@@ -97,7 +97,7 @@ void *thread_load_session(void *data)
 	}
 
 	/* Override existing session and autoload also. */
-	ret = config_load_session(info->path, NULL, 1, 1);
+	ret = config_load_session(info->path, NULL, 1, 1, NULL);
 	if (ret) {
 		ERR("Session load failed: %s", error_get_str(ret));
 	}
