@@ -26,6 +26,8 @@
 
 #include "consumer.h"
 #include "trace-kernel.h"
+#include "lttng-sessiond.h"
+#include "notification-thread-commands.h"
 
 /*
  * Find the channel name for the given kernel session.
@@ -179,6 +181,7 @@ struct ltt_kernel_channel *trace_kernel_create_channel(
 		struct lttng_channel *chan)
 {
 	struct ltt_kernel_channel *lkc;
+	struct lttng_channel_extended *extended = NULL;
 
 	assert(chan);
 
@@ -191,10 +194,18 @@ struct ltt_kernel_channel *trace_kernel_create_channel(
 	lkc->channel = zmalloc(sizeof(struct lttng_channel));
 	if (lkc->channel == NULL) {
 		PERROR("lttng_channel zmalloc");
-		free(lkc);
+		goto error;
+	}
+
+	extended = zmalloc(sizeof(struct lttng_channel_extended));
+	if (!extended) {
+		PERROR("lttng_channel_channel zmalloc");
 		goto error;
 	}
 	memcpy(lkc->channel, chan, sizeof(struct lttng_channel));
+	memcpy(extended, chan->attr.extended.ptr, sizeof(struct lttng_channel_extended));
+	lkc->channel->attr.extended.ptr = extended;
+	extended = NULL;
 
 	/*
 	 * If we receive an empty string for channel name, it means the
@@ -210,6 +221,7 @@ struct ltt_kernel_channel *trace_kernel_create_channel(
 	lkc->stream_count = 0;
 	lkc->event_count = 0;
 	lkc->enabled = 1;
+	lkc->published_to_notification_thread = false;
 	/* Init linked list */
 	CDS_INIT_LIST_HEAD(&lkc->events_list.head);
 	CDS_INIT_LIST_HEAD(&lkc->stream_list.head);
@@ -218,6 +230,11 @@ struct ltt_kernel_channel *trace_kernel_create_channel(
 	return lkc;
 
 error:
+	if (lkc) {
+		free(lkc->channel);
+	}
+	free(extended);
+	free(lkc);
 	return NULL;
 }
 
@@ -240,13 +257,35 @@ struct ltt_kernel_context *trace_kernel_create_context(
 	if (ctx) {
 		memcpy(&kctx->ctx, ctx, sizeof(kctx->ctx));
 	}
-
-	CDS_INIT_LIST_HEAD(&kctx->list);
-
 error:
 	return kctx;
 }
 
+/*
+ * Allocate and init a kernel context object from an existing kernel context
+ * object.
+ *
+ * Return the allocated object or NULL on error.
+ */
+struct ltt_kernel_context *trace_kernel_copy_context(
+		struct ltt_kernel_context *kctx)
+{
+	struct ltt_kernel_context *kctx_copy;
+
+	assert(kctx);
+	kctx_copy = zmalloc(sizeof(*kctx_copy));
+	if (!kctx_copy) {
+		PERROR("zmalloc ltt_kernel_context");
+		goto error;
+	}
+
+	memcpy(kctx_copy, kctx, sizeof(*kctx_copy));
+	memset(&kctx_copy->list, 0, sizeof(kctx_copy->list));
+
+error:
+	return kctx_copy;
+}
+
 /*
  * Allocate and initialize a kernel event. Set name and event type.
  * We own filter_expression, and filter.
@@ -462,7 +501,9 @@ void trace_kernel_destroy_context(struct ltt_kernel_context *ctx)
 {
 	assert(ctx);
 
-	cds_list_del(&ctx->list);
+	if (ctx->in_list) {
+		cds_list_del(&ctx->list);
+	}
 	free(ctx);
 }
 
@@ -475,6 +516,7 @@ void trace_kernel_destroy_channel(struct ltt_kernel_channel *channel)
 	struct ltt_kernel_event *event, *etmp;
 	struct ltt_kernel_context *ctx, *ctmp;
 	int ret;
+	enum lttng_error_code status;
 
 	assert(channel);
 
@@ -505,6 +547,14 @@ void trace_kernel_destroy_channel(struct ltt_kernel_channel *channel)
 	/* Remove from channel list */
 	cds_list_del(&channel->list);
 
+	if (notification_thread_handle
+			&& channel->published_to_notification_thread) {
+		status = notification_thread_command_remove_channel(
+				notification_thread_handle,
+				channel->fd, LTTNG_DOMAIN_KERNEL);
+		assert(status == LTTNG_OK);
+	}
+	free(channel->channel->attr.extended.ptr);
 	free(channel->channel);
 	free(channel);
 }
