@@ -51,6 +51,12 @@ static int opt_no_output;
 static int opt_snapshot;
 static unsigned int opt_live_timer;
 
+#ifdef LTTNG_EMBED_HELP
+static const char help_msg[] =
+#include <lttng-create.1.h>
+;
+#endif
+
 enum {
 	OPT_HELP = 1,
 	OPT_LIST_OPTIONS,
@@ -342,6 +348,10 @@ static int create_session(void)
 
 	/* Use default live URL if NO url is/are found. */
 	if ((opt_live_timer && !opt_url) && (opt_live_timer && !opt_data_url)) {
+		/* Override the url */
+		free(url);
+		url = NULL;
+
 		ret = asprintf(&alloc_url, "net://127.0.0.1");
 		if (ret < 0) {
 			PERROR("asprintf default live URL");
