@@ -15,7 +15,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define _GNU_SOURCE
 #define _LGPL_SOURCE
 #include <popt.h>
 #include <stdio.h>
@@ -24,6 +23,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <stdbool.h>
 
 #include "../command.h"
 
@@ -33,6 +33,13 @@
 
 static char *opt_session_name;
 static int opt_destroy_all;
+static int opt_no_wait;
+
+#ifdef LTTNG_EMBED_HELP
+static const char help_msg[] =
+#include <lttng-destroy.1.h>
+;
+#endif
 
 /* Mi writer */
 static struct mi_writer *writer;
@@ -47,26 +54,10 @@ static struct poptOption long_options[] = {
 	{"help",      'h', POPT_ARG_NONE, 0, OPT_HELP, 0, 0},
 	{"all",       'a', POPT_ARG_VAL, &opt_destroy_all, 1, 0, 0},
 	{"list-options", 0, POPT_ARG_NONE, NULL, OPT_LIST_OPTIONS, NULL, NULL},
+	{"no-wait",   'n', POPT_ARG_VAL, &opt_no_wait, 1, 0, 0},
 	{0, 0, 0, 0, 0, 0, 0}
 };
 
-/*
- * usage
- */
-static void usage(FILE *ofp)
-{
-	fprintf(ofp, "usage: lttng destroy [NAME] [OPTIONS]\n");
-	fprintf(ofp, "\n");
-	fprintf(ofp, "Where NAME is an optional session name. If not specified, lttng will\n");
-	fprintf(ofp, "get it from the configuration directory (.lttng).\n");
-	fprintf(ofp, "\n");
-	fprintf(ofp, "Options:\n");
-	fprintf(ofp, "  -h, --help           Show this help\n");
-	fprintf(ofp, "  -a, --all            Destroy all sessions\n");
-	fprintf(ofp, "      --list-options   Simple listing of options\n");
-	fprintf(ofp, "\n");
-}
-
 /*
  * destroy_session
  *
@@ -77,17 +68,53 @@ static int destroy_session(struct lttng_session *session)
 {
 	int ret;
 	char *session_name = NULL;
+	bool session_was_stopped;
 
-	ret = lttng_destroy_session(session->name);
-	if (ret < 0) {
-		switch (-ret) {
-		case LTTNG_ERR_SESS_NOT_FOUND:
-			WARN("Session name %s not found", session->name);
-			break;
-		default:
-			ERR("%s", lttng_strerror(ret));
-			break;
+	ret = lttng_stop_tracing_no_wait(session->name);
+	if (ret < 0 && ret != -LTTNG_ERR_TRACE_ALREADY_STOPPED) {
+		ERR("%s", lttng_strerror(ret));
+	}
+	session_was_stopped = ret == -LTTNG_ERR_TRACE_ALREADY_STOPPED;
+	if (!opt_no_wait) {
+		bool printed_wait_msg = false;
+
+		do {
+			ret = lttng_data_pending(session->name);
+			if (ret < 0) {
+				/* Return the data available call error. */
+				goto error;
+			}
+
+			/*
+			 * Data sleep time before retrying (in usec). Don't sleep if the call
+			 * returned value indicates availability.
+			 */
+			if (ret) {
+				if (!printed_wait_msg) {
+					_MSG("Waiting for data availability");
+					fflush(stdout);
+				}
+
+				printed_wait_msg = true;
+				usleep(DEFAULT_DATA_AVAILABILITY_WAIT_TIME);
+				_MSG(".");
+				fflush(stdout);
+			}
+		} while (ret != 0);
+		if (printed_wait_msg) {
+			MSG("");
 		}
+	}
+	if (!session_was_stopped) {
+		/*
+		 * Don't print the event and packet loss warnings since the user
+		 * already saw them when stopping the trace.
+		 */
+		print_session_stats(session->name);
+	}
+
+	ret = lttng_destroy_session_no_wait(session->name);
+	if (ret < 0) {
 		goto error;
 	}
 
@@ -158,13 +185,12 @@ int cmd_destroy(int argc, const char **argv)
 	while ((opt = poptGetNextOpt(pc)) != -1) {
 		switch (opt) {
 		case OPT_HELP:
-			usage(stdout);
+			SHOW_HELP();
 			break;
 		case OPT_LIST_OPTIONS:
 			list_cmd_options(stdout, long_options);
 			break;
 		default:
-			usage(stderr);
 			ret = CMD_UNDEFINED;
 			break;
 		}
@@ -206,7 +232,8 @@ int cmd_destroy(int argc, const char **argv)
 	/* Recuperate all sessions for further operation */
 	count = lttng_list_sessions(&sessions);
 	if (count < 0) {
-		command_ret = count;
+		ERR("%s", lttng_strerror(count));
+		command_ret = CMD_ERROR;
 		success = 0;
 		goto mi_closing;
 	}
