@@ -42,6 +42,12 @@ const char *indent4 = "    ";
 const char *indent6 = "      ";
 const char *indent8 = "        ";
 
+#ifdef LTTNG_EMBED_HELP
+static const char help_msg[] =
+#include <lttng-list.1.h>
+;
+#endif
+
 enum {
 	OPT_HELP = 1,
 	OPT_USERSPACE,
@@ -51,6 +57,9 @@ enum {
 static struct lttng_handle *handle;
 static struct mi_writer *writer;
 
+/* Only set when listing a single session. */
+static struct lttng_session listed_session;
+
 static struct poptOption long_options[] = {
 	/* longName, shortName, argInfo, argPtr, value, descrip, argDesc */
 	{"help",	'h', POPT_ARG_NONE, 0, OPT_HELP, 0, 0},
@@ -1142,7 +1151,7 @@ static int list_events(const char *channel_name)
 		}
 	} else {
 		/* Pretty print */
-		MSG("\n%sEvents:", indent4);
+		MSG("\n%sEvent rules:", indent4);
 		if (count == 0) {
 			MSG("%sNone\n", indent6);
 			goto end;
@@ -1160,13 +1169,31 @@ error:
 	return ret;
 }
 
+static
+void print_timer(const char *timer_name, uint32_t space_count, int64_t value)
+{
+	uint32_t i;
+
+	_MSG("%s%s:", indent6, timer_name);
+	for (i = 0; i < space_count; i++) {
+		_MSG(" ");
+	}
+
+	if (value) {
+		MSG("%" PRId64 " µs", value);
+	} else {
+		MSG("inactive");
+	}
+}
+
 /*
  * Pretty print channel
  */
 static void print_channel(struct lttng_channel *channel)
 {
 	int ret;
-	uint64_t discarded_events, lost_packets;
+	uint64_t discarded_events, lost_packets, monitor_timer_interval;
+	int64_t blocking_timeout;
 
 	ret = lttng_channel_get_discarded_event_count(channel,
 			&discarded_events);
@@ -1182,26 +1209,82 @@ static void print_channel(struct lttng_channel *channel)
 		return;
 	}
 
-	MSG("- %s:%s\n", channel->name, enabled_string(channel->enabled));
+	ret = lttng_channel_get_monitor_timer_interval(channel,
+			&monitor_timer_interval);
+	if (ret) {
+		ERR("Failed to retrieve monitor interval of channel");
+		return;
+	}
 
+	ret = lttng_channel_get_blocking_timeout(channel,
+			&blocking_timeout);
+	if (ret) {
+		ERR("Failed to retrieve blocking timeout of channel");
+		return;
+	}
+
+	MSG("- %s:%s\n", channel->name, enabled_string(channel->enabled));
 	MSG("%sAttributes:", indent4);
-	MSG("%soverwrite mode: %d", indent6, channel->attr.overwrite);
-	MSG("%ssubbufers size: %" PRIu64, indent6, channel->attr.subbuf_size);
-	MSG("%snumber of subbufers: %" PRIu64, indent6, channel->attr.num_subbuf);
-	MSG("%sswitch timer interval: %u", indent6, channel->attr.switch_timer_interval);
-	MSG("%sread timer interval: %u", indent6, channel->attr.read_timer_interval);
-	MSG("%strace file count: %" PRIu64, indent6, channel->attr.tracefile_count);
-	MSG("%strace file size (bytes): %" PRIu64, indent6, channel->attr.tracefile_size);
-	MSG("%sdiscarded events: %" PRIu64, indent6, discarded_events);
-	MSG("%slost packets: %" PRIu64, indent6, lost_packets);
+	MSG("%sEvent-loss mode:  %s", indent6, channel->attr.overwrite ? "overwrite" : "discard");
+	MSG("%sSub-buffer size:  %" PRIu64 " bytes", indent6, channel->attr.subbuf_size);
+	MSG("%sSub-buffer count: %" PRIu64, indent6, channel->attr.num_subbuf);
+
+	print_timer("Switch timer", 5, channel->attr.switch_timer_interval);
+	print_timer("Read timer",  7, channel->attr.read_timer_interval);
+	print_timer("Monitor timer", 4, monitor_timer_interval);
+
+	if (!channel->attr.overwrite) {
+		if (blocking_timeout == -1) {
+			MSG("%sBlocking timeout: infinite", indent6);
+		} else {
+			MSG("%sBlocking timeout: %" PRId64 " µs", indent6, blocking_timeout);
+		}
+	}
+
+	MSG("%sTrace file count: %" PRIu64 " per stream", indent6,
+			channel->attr.tracefile_count == 0 ?
+				1 : channel->attr.tracefile_count);
+	if (channel->attr.tracefile_size != 0 ) {
+		MSG("%sTrace file size:  %" PRIu64 " bytes", indent6,
+				channel->attr.tracefile_size);
+	} else {
+		MSG("%sTrace file size:  %s", indent6, "unlimited");
+	}
 	switch (channel->attr.output) {
 		case LTTNG_EVENT_SPLICE:
-			MSG("%soutput: splice()", indent6);
+			MSG("%sOutput mode:      splice", indent6);
 			break;
 		case LTTNG_EVENT_MMAP:
-			MSG("%soutput: mmap()", indent6);
+			MSG("%sOutput mode:      mmap", indent6);
 			break;
 	}
+
+	MSG("\n%sStatistics:", indent4);
+	if (listed_session.snapshot_mode) {
+		/*
+		 * The lost packet count is omitted for sessions in snapshot
+		 * mode as it is misleading: it would indicate the number of
+		 * packets that the consumer could not extract during the
+		 * course of recording the snapshot. It does not have the
+		 * same meaning as the "regular" lost packet count that
+		 * would result from the consumer not keeping up with
+		 * event production in an overwrite-mode channel.
+		 *
+		 * A more interesting statistic would be the number of
+		 * packets lost between the first and last extracted
+		 * packets of a given snapshot (which prevents most analyses).
+		 */
+		MSG("%sNone", indent6);
+		goto skip_stats_printing;
+	}
+
+	if (!channel->attr.overwrite) {
+		MSG("%sDiscarded events: %" PRIu64, indent6, discarded_events);
+	} else {
+		MSG("%sLost packets:     %" PRIu64, indent6, lost_packets);
+	}
+skip_stats_printing:
+	return;
 }
 
 /*
@@ -1561,6 +1644,8 @@ static int list_sessions(const char *session_name)
 							active_string(sessions[i].enabled),
 							snapshot_string(sessions[i].snapshot_mode));
 					MSG("%sTrace path: %s\n", indent4, sessions[i].path);
+					memcpy(&listed_session, &sessions[i],
+							sizeof(listed_session));
 					break;
 				}
 			} else {
@@ -1570,7 +1655,7 @@ static int list_sessions(const char *session_name)
 						snapshot_string(sessions[i].snapshot_mode));
 				MSG("%sTrace path: %s", indent4, sessions[i].path);
 				if (sessions[i].live_timer_interval != 0) {
-					MSG("%sLive timer interval (usec): %u", indent4,
+					MSG("%sLive timer interval: %u µs", indent4,
 							sessions[i].live_timer_interval);
 				}
 				MSG("");
