@@ -53,6 +53,11 @@ enum lttng_kernel_context_type {
 	LTTNG_KERNEL_CONTEXT_PPID           = 8,
 	LTTNG_KERNEL_CONTEXT_VPPID          = 9,
 	LTTNG_KERNEL_CONTEXT_HOSTNAME       = 10,
+	LTTNG_KERNEL_CONTEXT_CPU_ID         = 11,
+	LTTNG_KERNEL_CONTEXT_INTERRUPTIBLE  = 12,
+	LTTNG_KERNEL_CONTEXT_PREEMPTIBLE    = 13,
+	LTTNG_KERNEL_CONTEXT_NEED_RESCHEDULE = 14,
+	LTTNG_KERNEL_CONTEXT_MIGRATABLE     = 15,
 };
 
 /* Perf counter attributes */
@@ -124,14 +129,6 @@ struct lttng_kernel_tracer_abi_version {
 	uint32_t minor;
 } LTTNG_PACKED;
 
-enum lttng_kernel_calibrate_type {
-	LTTNG_KERNEL_CALIBRATE_KRETPROBE,
-};
-
-struct lttng_kernel_calibrate {
-	enum lttng_kernel_calibrate_type type;	/* type (input) */
-} LTTNG_PACKED;
-
 struct lttng_kernel_syscall_mask {
 	uint32_t len;	/* in bits */
 	char mask[];
