@@ -47,7 +47,6 @@ LTTNG_HIDDEN const char * const mi_lttng_schema_version_value = XSTR(MI_SCHEMA_M
 const char * const mi_lttng_element_command = "command";
 const char * const mi_lttng_element_command_action = "snapshot_action";
 const char * const mi_lttng_element_command_add_context = "add-context";
-const char * const mi_lttng_element_command_calibrate = "calibrate";
 const char * const mi_lttng_element_command_create = "create";
 const char * const mi_lttng_element_command_destroy = "destroy";
 const char * const mi_lttng_element_command_disable_channel = "disable-channel";
@@ -58,6 +57,8 @@ const char * const mi_lttng_element_command_list = "list";
 const char * const mi_lttng_element_command_load = "load";
 LTTNG_HIDDEN const char * const mi_lttng_element_command_metadata = "metadata";
 LTTNG_HIDDEN const char * const mi_lttng_element_command_metadata_action = "metadata_action";
+LTTNG_HIDDEN const char * const mi_lttng_element_command_regenerate = "regenerate";
+LTTNG_HIDDEN const char * const mi_lttng_element_command_regenerate_action = "regenerate_action";
 const char * const mi_lttng_element_command_name = "name";
 const char * const mi_lttng_element_command_output = "output";
 const char * const mi_lttng_element_command_save = "save";
@@ -100,6 +101,8 @@ const char * const mi_lttng_element_save = "save";
 
 /* Strings related to load command */
 const char * const mi_lttng_element_load = "load";
+LTTNG_HIDDEN const char * const mi_lttng_element_load_overrides = "overrides";
+LTTNG_HIDDEN const char * const mi_lttng_element_load_override_url = "url";
 
 /* General elements of mi_lttng */
 const char * const mi_lttng_element_empty = "";
@@ -165,10 +168,6 @@ const char * const mi_lttng_loglevel_type_range = "RANGE";
 const char * const mi_lttng_loglevel_type_single = "SINGLE";
 const char * const mi_lttng_loglevel_type_unknown = "UNKNOWN";
 
-/* String related to lttng_calibrate */
-const char * const mi_lttng_element_calibrate = "calibrate";
-const char * const mi_lttng_element_calibrate_function = "FUNCTION";
-
 /* String related to a lttng_snapshot_output */
 const char * const mi_lttng_element_snapshot_ctrl_url = "ctrl_url";
 const char * const mi_lttng_element_snapshot_data_url = "data_url";
@@ -186,6 +185,9 @@ const char * const mi_lttng_context_type_perf_cpu_counter;
 const char * const mi_lttng_context_type_perf_thread_counter;
 const char * const mi_lttng_element_track_untrack_pid_target;
 const char * const mi_lttng_element_track_untrack_targets;
+const char * const mi_lttng_element_calibrate;
+const char * const mi_lttng_element_calibrate_function;
+const char * const mi_lttng_element_command_calibrate;
 
 /* This is a merge of jul loglevel and regular loglevel
  * Those should never overlap by definition
@@ -434,22 +436,6 @@ const char *mi_lttng_buffertype_string(enum lttng_buffer_type value)
 	}
 }
 
-LTTNG_HIDDEN
-const char *mi_lttng_calibratetype_string(enum lttng_calibrate_type val)
-{
-	const char *ret;
-
-	switch (val) {
-	case LTTNG_CALIBRATE_FUNCTION:
-		ret = mi_lttng_element_calibrate_function;
-		break;
-	default:
-		ret = mi_lttng_element_empty;
-		break;
-	}
-	return ret;
-}
-
 LTTNG_HIDDEN
 struct mi_writer *mi_lttng_writer_create(int fd_output, int mi_output_type)
 {
@@ -876,7 +862,8 @@ int mi_lttng_channel_attr(struct mi_writer *writer,
 	int ret = 0;
 	struct lttng_channel *chan = caa_container_of(attr,
 			struct lttng_channel, attr);
-	uint64_t discarded_events, lost_packets;
+	uint64_t discarded_events, lost_packets, monitor_timer_interval;
+	int64_t blocking_timeout;
 
 	assert(attr);
 
@@ -890,6 +877,18 @@ int mi_lttng_channel_attr(struct mi_writer *writer,
 		goto end;
 	}
 
+	ret = lttng_channel_get_monitor_timer_interval(chan,
+			&monitor_timer_interval);
+	if (ret) {
+		goto end;
+	}
+
+	ret = lttng_channel_get_blocking_timeout(chan,
+			&blocking_timeout);
+	if (ret) {
+		goto end;
+	}
+
 	/* Opening Attributes */
 	ret = mi_lttng_writer_open_element(writer, config_element_attributes);
 	if (ret) {
@@ -936,6 +935,22 @@ int mi_lttng_channel_attr(struct mi_writer *writer,
 		goto end;
 	}
 
+	/* Monitor timer interval in usec */
+	ret = mi_lttng_writer_write_element_unsigned_int(writer,
+		config_element_monitor_timer_interval,
+		monitor_timer_interval);
+	if (ret) {
+		goto end;
+	}
+
+	/* Retry timeout in usec */
+	ret = mi_lttng_writer_write_element_signed_int(writer,
+		config_element_blocking_timeout,
+		blocking_timeout);
+	if (ret) {
+		goto end;
+	}
+
 	/* Event output */
 	ret = mi_lttng_writer_write_element_string(writer,
 		config_element_output_type,
@@ -1423,31 +1438,6 @@ end:
 	return ret;
 }
 
-LTTNG_HIDDEN
-int mi_lttng_calibrate(struct mi_writer *writer,
-		struct lttng_calibrate *calibrate)
-{
-	int ret;
-
-	/* Open calibrate element */
-	ret = mi_lttng_writer_open_element(writer, mi_lttng_element_calibrate);
-	if (ret) {
-		goto end;
-	}
-
-	/* Calibration type */
-	ret = mi_lttng_writer_write_element_string(writer, config_element_type,
-			mi_lttng_calibratetype_string(calibrate->type));
-	if (ret) {
-		goto end;
-	}
-
-	/* Closing calibrate element */
-	ret = mi_lttng_writer_close_element(writer);
-end:
-	return ret;
-}
-
 LTTNG_HIDDEN
 int mi_lttng_perf_counter_context(struct mi_writer *writer,
 		struct lttng_event_perf_counter_ctx  *perf_context)
