@@ -329,7 +329,6 @@ int run_as_worker(struct run_as_worker *worker)
 	if (ret && ret != -ENOSYS) {
 		/* Don't fail as this is not essential. */
 		PERROR("prctl PR_SET_NAME");
-		ret = 0;
 	}
 
 	sendret.ret = 0;
@@ -466,6 +465,7 @@ int run_as_mkdir_recursive(const char *path, mode_t mode, uid_t uid, gid_t gid)
 {
 	struct run_as_data data;
 
+	memset(&data, 0, sizeof(data));
 	DBG3("mkdir() recursive %s with mode %d for uid %d and gid %d",
 			path, (int) mode, (int) uid, (int) gid);
 	strncpy(data.u.mkdir.path, path, PATH_MAX - 1);
@@ -479,6 +479,7 @@ int run_as_mkdir(const char *path, mode_t mode, uid_t uid, gid_t gid)
 {
 	struct run_as_data data;
 
+	memset(&data, 0, sizeof(data));
 	DBG3("mkdir() %s with mode %d for uid %d and gid %d",
 			path, (int) mode, (int) uid, (int) gid);
 	strncpy(data.u.mkdir.path, path, PATH_MAX - 1);
@@ -487,15 +488,12 @@ int run_as_mkdir(const char *path, mode_t mode, uid_t uid, gid_t gid)
 	return run_as(RUN_AS_MKDIR, &data, uid, gid);
 }
 
-/*
- * Note: open_run_as is currently not working. We'd need to pass the fd
- * opened in the child to the parent.
- */
 LTTNG_HIDDEN
 int run_as_open(const char *path, int flags, mode_t mode, uid_t uid, gid_t gid)
 {
 	struct run_as_data data;
 
+	memset(&data, 0, sizeof(data));
 	DBG3("open() %s with flags %X mode %d for uid %d and gid %d",
 			path, flags, (int) mode, (int) uid, (int) gid);
 	strncpy(data.u.open.path, path, PATH_MAX - 1);
@@ -510,6 +508,7 @@ int run_as_unlink(const char *path, uid_t uid, gid_t gid)
 {
 	struct run_as_data data;
 
+	memset(&data, 0, sizeof(data));
 	DBG3("unlink() %s with for uid %d and gid %d",
 			path, (int) uid, (int) gid);
 	strncpy(data.u.unlink.path, path, PATH_MAX - 1);
