@@ -34,12 +34,6 @@ int run_as_unlink(const char *path, uid_t uid, gid_t gid);
 LTTNG_HIDDEN
 int run_as_rmdir_recursive(const char *path, uid_t uid, gid_t gid);
 
-/* Backward compat. */
-static inline int run_as_recursive_rmdir(const char *path, uid_t uid, gid_t gid)
-{
-	return run_as_rmdir_recursive(path, uid, gid);
-}
-
 LTTNG_HIDDEN
 int run_as_create_worker(char *procname);
 LTTNG_HIDDEN
