@@ -94,13 +94,6 @@ struct lttcomm_relayd_generic_reply {
 	uint32_t ret_code;
 } LTTNG_PACKED;
 
-/*
- * Used to update synchronization information.
- */
-struct lttcomm_relayd_update_sync_info {
-	/* TODO: fill the structure. Feature not implemented yet */
-} LTTNG_PACKED;
-
 /*
  * Version command.
  */
@@ -159,10 +152,30 @@ struct lttcomm_relayd_index {
 	uint64_t timestamp_end;
 	uint64_t events_discarded;
 	uint64_t stream_id;
+	/* 2.8+ */
 	uint64_t stream_instance_id;
 	uint64_t packet_seq_num;
 } LTTNG_PACKED;
 
+static inline size_t lttcomm_relayd_index_len(uint32_t major, uint32_t minor)
+{
+	if (major == 1) {
+		switch (minor) {
+		case 0:
+			return offsetof(struct lttcomm_relayd_index, stream_id)
+				+ member_sizeof(struct lttcomm_relayd_index,
+						stream_id);
+		case 1:
+			return offsetof(struct lttcomm_relayd_index, packet_seq_num)
+				+ member_sizeof(struct lttcomm_relayd_index,
+						packet_seq_num);
+		default:
+			abort();
+		}
+	}
+	abort();
+}
+
 /*
  * Create session in 2.4 adds additionnal parameters for live reading.
  */
