@@ -23,6 +23,7 @@ import java.lang.Integer;
 import org.apache.log4j.Appender;
 import org.apache.log4j.BasicConfigurator;
 import org.apache.log4j.Logger;
+import org.apache.log4j.Level;
 import org.lttng.ust.agent.log4j.LttngLogAppender;
 
 public class JTestLTTng {
@@ -40,6 +41,18 @@ public class JTestLTTng {
 		Logger lttng = Logger.getLogger("log4j-event");
 		Logger lttng2 = Logger.getLogger("log4j-event-2");
 
+		/*
+		 * Set lowest level to make sure all event levels are logged.
+		 * Any jar can override the default log4j rootLogger level
+		 * and a logger with no explicit level defaults to the non-null
+		 * parent level. Events could be ignored if the inherited value
+		 * is too low, thereby failing the test.
+		 *
+		 * See BSF  -> https://issues.apache.org/jira/browse/BSF-24
+		 */
+		lttng.setLevel(Level.ALL);
+		lttng2.setLevel(Level.ALL);
+
 		int nrIter = Integer.parseInt(args[0]);
 		int waitTime = Integer.parseInt(args[1]);
 		int fire_debug_tp = 0;
