@@ -38,6 +38,7 @@
 int lttng_opt_quiet = 1;
 int lttng_opt_verbose;
 int lttng_opt_mi;
+struct notification_thread_handle *notification_thread_handle;
 
 int ust_consumerd32_fd;
 int ust_consumerd64_fd;
@@ -72,6 +73,10 @@ static void test_create_one_kernel_session(void)
 	kern = trace_kernel_create_session();
 	ok(kern != NULL, "Create kernel session");
 
+	if (!kern) {
+		skip(1, "Kernel session is null");
+		return;
+	}
 	ok(kern->fd == -1 &&
 	   kern->metadata_stream_fd == -1 &&
 	   kern->consumer_fds_sent == 0 &&
@@ -111,12 +116,20 @@ static void test_create_kernel_channel(void)
 {
 	struct ltt_kernel_channel *chan;
 	struct lttng_channel attr;
+	struct lttng_channel_extended extended;
 
 	memset(&attr, 0, sizeof(attr));
+	memset(&extended, 0, sizeof(extended));
+	attr.attr.extended.ptr = &extended;
 
 	chan = trace_kernel_create_channel(&attr);
 	ok(chan != NULL, "Create kernel channel");
 
+	if (!chan) {
+		skip(1, "Channel is null");
+		return;
+	}
+
 	ok(chan->fd == -1 &&
 	   chan->enabled == 1 &&
 	   chan->stream_count == 0 &&
@@ -143,6 +156,11 @@ static void test_create_kernel_event(void)
 	event = trace_kernel_create_event(&ev, NULL, NULL);
 	ok(event != NULL, "Create kernel event");
 
+	if (!event) {
+		skip(1, "Event is null");
+		return;
+	}
+
 	ok(event->fd == -1 &&
 	   event->enabled == 1 &&
 	   event->event->instrumentation == LTTNG_KERNEL_TRACEPOINT &&
@@ -161,6 +179,11 @@ static void test_create_kernel_stream(void)
 	stream = trace_kernel_create_stream("stream1", 0);
 	ok(stream != NULL, "Create kernel stream");
 
+	if (!stream) {
+		skip(1, "Stream is null");
+		return;
+	}
+
 	ok(stream->fd == -1 &&
 	   stream->state == 0,
 	   "Validate kernel stream");
