@@ -24,6 +24,7 @@
 #include <unistd.h>
 #include <time.h>
 #include <sys/types.h>
+#include <urcu.h>
 
 #include <tap/tap.h>
 
@@ -302,6 +303,8 @@ int main(int argc, char **argv)
 
 	diag("Sessions unit tests");
 
+	rcu_register_thread();
+
 	test_session_list();
 
 	test_create_one_session();
@@ -318,6 +321,7 @@ int main(int argc, char **argv)
 
 	test_large_session_number();
 
+	rcu_unregister_thread();
 	assert(!fini_ht_cleanup_thread(&ht_cleanup_thread));
 
 	return exit_status();
